@@ -9,15 +9,21 @@ Everything relating to the 'project context' object that describes the project f
 
 import os
 import copy
-import pytomlpp
+try:
+	import pytomlpp as toml  # fast; based on toml++ (C++)
+except:
+	try:
+		import tomllib as toml  # PEP 680
+	except:
+		import tomli as toml
 import datetime
-import shutil
 import itertools
 from schema import Schema, Or, And, Optional
 from .utils import *
 from . import dirs
 from . import repos
 from . import emoji
+from . import doxygen
 
 #=======================================================================================================================
 # schemas
@@ -807,8 +813,6 @@ class Defaults(object):
 		r'std::(?:literals::)?(?:chrono|complex|string|string_view)_literals', r'std::literals', r'std::numbers',
 		r'std::ranges', r'std::this_thread'
 	}
-	cb_numeric_literals = set()
-	cb_string_literals = {r'sv?'}
 	cb_types = {
 		#------ built-in types
 		r'__(?:float|fp)[0-9]{1,3}',
@@ -919,8 +923,8 @@ class CodeBlocks(object):
 	schema = {
 		Optional(r'types'): ValueOrArray(str, name=r'types'),
 		Optional(r'macros'): ValueOrArray(str, name=r'macros'),
-		Optional(r'string_literals'): ValueOrArray(str, name=r'string_literals'),
-		Optional(r'numeric_literals'): ValueOrArray(str, name=r'numeric_literals'),
+		Optional(r'string_literals'): ValueOrArray(str, name=r'string_literals'),  # deprecated
+		Optional(r'numeric_literals'): ValueOrArray(str, name=r'numeric_literals'),  # deprecated
 		Optional(r'enums'): ValueOrArray(str, name=r'enums'),
 		Optional(r'namespaces'): ValueOrArray(str, name=r'namespaces'),
 	}
@@ -928,8 +932,6 @@ class CodeBlocks(object):
 	def __init__(self, config, macros):
 		self.types = copy.deepcopy(Defaults.cb_types)
 		self.macros = copy.deepcopy(Defaults.cb_macros)
-		self.string_literals = copy.deepcopy(Defaults.cb_string_literals)
-		self.numeric_literals = copy.deepcopy(Defaults.cb_numeric_literals)
 		self.enums = copy.deepcopy(Defaults.cb_enums)
 		self.namespaces = copy.deepcopy(Defaults.cb_namespaces)
 
@@ -948,18 +950,6 @@ class CodeBlocks(object):
 					if macro:
 						self.macros.add(macro)
 
-			if 'string_literals' in config:
-				for lit in coerce_collection(config['string_literals']):
-					literal = lit.strip()
-					if literal:
-						self.string_literals.add(literal)
-
-			if 'numeric_literals' in config:
-				for lit in coerce_collection(config['numeric_literals']):
-					literal = lit.strip()
-					if literal:
-						self.numeric_literals.add(literal)
-
 			if 'enums' in config:
 				for e in coerce_collection(config['enums']):
 					enum = e.strip()
@@ -1265,13 +1255,27 @@ class Context(object):
 				self.verbose_value(rf'{name}.{k}', v)
 
 	def __init__(
-		self, config_path, output_dir, threads, cleanup, verbose, doxygen_path, logger, xml_only, html_include,
-		html_exclude, treat_warnings_as_errors, theme, copy_assets
+		self,  #
+		config_path: Path,
+		output_dir: Path,
+		output_html: bool,
+		output_xml: bool,
+		threads: int,
+		cleanup: bool,
+		verbose: bool,
+		logger,
+		html_include: str,
+		html_exclude: str,
+		treat_warnings_as_errors: bool,
+		theme: str,
+		copy_assets: bool,
+		**kwargs
 	):
 
 		self.logger = logger
 		self.__verbose = bool(verbose)
-		self.xml_only = bool(xml_only)
+		self.output_html = bool(output_html)
+		self.output_xml = bool(output_xml)
 		self.cleanup = bool(cleanup)
 		self.copy_assets = bool(copy_assets)
 		self.verbose_logger = logger if self.__verbose else None
@@ -1280,7 +1284,8 @@ class Context(object):
 		self.version_string = r'.'.join([str(v) for v in lib_version()])
 		self.info(rf'Poxy v{self.version_string}')
 
-		self.verbose_value(r'Context.xml_only', self.xml_only)
+		self.verbose_value(r'Context.output_html', self.output_html)
+		self.verbose_value(r'Context.output_xml', self.output_xml)
 		self.verbose_value(r'Context.cleanup', self.cleanup)
 
 		threads = int(threads) if threads is not None else 0
@@ -1289,15 +1294,21 @@ class Context(object):
 		self.threads = max(1, min(os.cpu_count(), threads))
 		self.verbose_value(r'Context.threads', self.threads)
 
-		self.fixers = None
-		self.tagfile_path = None
+		# additional kwargs (experimental stuff etc)
+		self.xml_v2 = bool(kwargs[r'xml_v2']) if r'xml_v2' in kwargs else False
+		self.verbose_value(r'Context.xml_v2', self.xml_v2)
+
+		# these are overridden/initialized elsewhere; they're here so duck-typing still quacks
+		self.fixers = []
+		self.compound_pages = dict()
+		self.compound_kinds = set()
 
 		# initial warning state
 		# note that this is overwritten after the config is read;
 		# it is set here first so that we can have correct 'treat_as_errors' behaviour if we add any pre-config warnings
 		self.warnings = Warnings(None)
-		if treat_warnings_as_errors:
-			self.warnings.treat_as_errors = True
+		if treat_warnings_as_errors is not None:
+			self.warnings.treat_as_errors = bool(treat_warnings_as_errors)
 
 		if html_include is not None:
 			html_include = re.compile(str(html_include))
@@ -1315,6 +1326,7 @@ class Context(object):
 		self.verbose_value(r'dirs.GENERATED', dirs.GENERATED)
 		self.verbose_value(r'dirs.FONTS', dirs.FONTS)
 		self.verbose_value(r'dirs.TEMP', dirs.TEMP)
+		self.verbose_value(r'doxygen.path()', doxygen.path())
 
 		# resolve paths
 		if 1:
@@ -1347,23 +1359,7 @@ class Context(object):
 			assert_existing_directory(self.input_dir)
 			assert self.input_dir.is_absolute()
 
-			# output paths
-			self.xml_dir = Path(self.output_dir, r'xml')
-			self.verbose_value(r'Context.xml_dir', self.xml_dir)
-			self.html_dir = Path(self.output_dir, r'html')
-			self.assets_dir = Path(self.html_dir, r'poxy')
-			self.verbose_value(r'Context.html_dir', self.html_dir)
-			self.verbose_value(r'Context.assets_dir', self.assets_dir)
-			assert self.output_dir.is_absolute()
-			assert self.xml_dir.is_absolute()
-			assert self.html_dir.is_absolute()
-
-			# blog dir
-			self.blog_dir = Path(self.input_dir, r'blog')
-			self.verbose_value(r'Context.blog_dir', self.blog_dir)
-			assert self.blog_dir.is_absolute()
-
-			# temp dirs
+			# root temp dir for this run
 			self.temp_dir = re.sub(r'''[!@#$%^&*()+={}<>;:'"_\\/\n\t -]+''', r'_', str(self.input_dir).strip(r'\/'))
 			if len(self.temp_dir) > 256:
 				self.temp_dir = str(self.input_dir)
@@ -1372,53 +1368,45 @@ class Context(object):
 				self.temp_dir = sha1(self.temp_dir)
 			self.temp_dir = Path(dirs.TEMP, self.temp_dir)
 			self.verbose_value(r'Context.temp_dir', self.temp_dir)
-			self.temp_pages_dir = Path(self.temp_dir, r'pages')
-			self.verbose_value(r'Context.temp_pages_dir', self.temp_pages_dir)
 			assert self.temp_dir.is_absolute()
+
+			# temp pages dir
+			self.temp_pages_dir = Path(self.temp_dir, r'pages')
+			self.verbose_value(r'Context.pages_dir', self.temp_pages_dir)
 			assert self.temp_pages_dir.is_absolute()
 
-			# delete leftovers from previous run and initialize various dirs
-			delete_directory(self.xml_dir, logger=self.verbose_logger)
+			# temp xml output path used by doxygen
+			self.temp_xml_dir = Path(self.temp_dir, r'xml')
+			self.verbose_value(r'Context.temp_xml_dir', self.temp_xml_dir)
+			assert self.temp_xml_dir.is_absolute()
+
+			# xml output path (--xml)
+			self.xml_dir = Path(self.output_dir, r'xml')
+			self.verbose_value(r'Context.xml_dir', self.xml_dir)
+			assert self.xml_dir.is_absolute()
+
+			# html output path (--xml)
+			self.html_dir = Path(self.output_dir, r'html')
+			self.verbose_value(r'Context.html_dir', self.html_dir)
+			assert self.html_dir.is_absolute()
+
+			# assets subdir in html output
+			self.assets_dir = Path(self.html_dir, r'poxy')
+			self.verbose_value(r'Context.assets_dir', self.assets_dir)
+			assert self.assets_dir.is_absolute()
+
+			# blog dir
+			self.blog_dir = Path(self.input_dir, r'blog')
+			self.verbose_value(r'Context.blog_dir', self.blog_dir)
+			assert self.blog_dir.is_absolute()
+
+			# delete leftovers from previous run and initialize temp dirs
 			delete_directory(self.temp_dir, logger=self.verbose_logger)
-			if not self.xml_only:
-				delete_directory(self.html_dir, logger=self.verbose_logger)
+			delete_directory(self.xml_dir, logger=self.verbose_logger)
+			delete_directory(self.html_dir, logger=self.verbose_logger)
 			self.temp_dir.mkdir(exist_ok=True, parents=True)
 			self.temp_pages_dir.mkdir(exist_ok=True, parents=True)
 
-			# doxygen
-			if doxygen_path is not None:
-				doxygen_path = coerce_path(doxygen_path).resolve()
-				if not doxygen_path.exists() and Path(str(doxygen_path) + r'.exe').exists():
-					doxygen_path = Path(str(doxygen_path) + r'.exe')
-			else:
-				doxygen_path = shutil.which(r'doxygen')
-				if doxygen_path is not None:
-					doxygen_path = coerce_path(doxygen_path)
-				if doxygen_path is None or not doxygen_path.exists():
-					try:
-						doxygen_path = Path(
-							'C:\\Program Files\\doxygen\\bin\\doxygen.exe'
-						)  # todo: get 'program files' path programatically
-					except:  # path will be invalid on non-windows
-						pass
-				if doxygen_path is None or not doxygen_path.exists():
-					raise Error(rf'Could not find Doxygen on system path')
-			assert doxygen_path is not None
-			doxygen_path = coerce_path(doxygen_path).resolve()
-			if doxygen_path.is_dir():
-				p = Path(doxygen_path, r'doxygen.exe')
-				if not p.exists() or not p.is_file() or not os.access(str(p), os.X_OK):
-					p = Path(doxygen_path, r'doxygen')
-				if not p.exists() or not p.is_file() or not os.access(str(p), os.X_OK):
-					raise Error(rf'Could not find Doxygen executable in {doxygen_path}')
-				doxygen_path = p
-			assert_existing_file(doxygen_path)
-			if not os.access(str(doxygen_path), os.X_OK):
-				raise Error(rf'{doxygen_path} was not an executable file')
-			self.doxygen_path = doxygen_path
-			self.verbose_value(r'Context.doxygen_path', self.doxygen_path)
-			assert self.doxygen_path.is_absolute()
-
 			# temp doxyfile path
 			self.doxyfile_path = Path(self.temp_dir, rf'Doxyfile')
 			self.verbose_value(r'Context.doxyfile_path', self.doxyfile_path)
@@ -1455,12 +1443,12 @@ class Context(object):
 			config = dict()
 			if self.config_path.exists():
 				assert_existing_file(self.config_path)
-				config = pytomlpp.loads(read_all_text_from_file(self.config_path, logger=self.logger))
+				config = toml.loads(read_all_text_from_file(self.config_path, logger=self.logger))
 			config = assert_no_unexpected_keys(config, self.__config_schema.validate(config))
 
 			self.warnings = Warnings(config)
-			if treat_warnings_as_errors:
-				self.warnings.treat_as_errors = True
+			if treat_warnings_as_errors is not None:
+				self.warnings.treat_as_errors = bool(treat_warnings_as_errors)
 			self.verbose_value(r'Context.warnings', self.warnings)
 
 			# project name (PROJECT_NAME)
@@ -1633,12 +1621,15 @@ class Context(object):
 			if r'changelog' in config:
 				if isinstance(config['changelog'], bool):
 					if config['changelog']:
-						candidate_names = (r'CHANGELOG', r'HISTORY', r'changelog', r'history')
+						candidate_names = (r'CHANGELOG', r'CHANGES', r'HISTORY')
 						candidate_extensions = (r'.md', r'.txt', r'')
+						as_lowercase = (False, True)
 						candidate_dir = self.input_dir
 						while True:
-							for name, ext in itertools.product(candidate_names, candidate_extensions):
-								candidate_file = Path(candidate_dir, rf'{name}{ext}')
+							for name, ext, lower in itertools.product(
+								candidate_names, candidate_extensions, as_lowercase
+							):
+								candidate_file = Path(candidate_dir, rf'{name.lower() if lower else name}{ext}')
 								if candidate_file.exists() and candidate_file.is_file(
 								) and candidate_file.stat().st_size <= 1024 * 1024 * 2:
 									self.changelog = candidate_file
@@ -1837,9 +1828,12 @@ class Context(object):
 			self.verbose_value(r'Context.internal_docs', self.internal_docs)
 
 			# generate_tagfile (GENERATE_TAGFILE)
-			self.generate_tagfile = True  # not (self.private_repo or self.internal_docs)
-			if 'generate_tagfile' in config:
-				self.generate_tagfile = bool(config['generate_tagfile'])
+			self.generate_tagfile = True
+			self.tagfile_path = Path(
+				self.temp_dir, rf'{self.name.replace(" ","_")}.tagfile.xml' if self.name else r'tagfile.xml'
+			)
+			if r'generate_tagfile' in config:
+				self.generate_tagfile = bool(config[r'generate_tagfile'])
 			self.verbose_value(r'Context.generate_tagfile', self.generate_tagfile)
 
 			# favicon (M_FAVICON)
@@ -1966,8 +1960,6 @@ class Context(object):
 	def __exit__(self, type, value, traceback):
 		if self.cleanup:
 			delete_directory(self.temp_dir, logger=self.verbose_logger)
-			if not self.xml_only:
-				delete_directory(self.xml_dir, logger=self.verbose_logger)
 
 	def __bool__(self):
 		return True
