@@ -18,6 +18,7 @@ from .project import Context
 from . import doxygen
 from . import soup
 from . import fixers
+from . import graph
 from .svg import SVG
 from distutils.dir_util import copy_tree
 
@@ -36,7 +37,7 @@ def make_temp_file():
 # PRE/POST PROCESSORS
 #=======================================================================================================================
 
-_doxygen_overrides = (
+DOXYGEN_DEFAULTS = (
 	(r'ALLEXTERNALS', False),
 	(r'ALLOW_UNICODE_NAMES', False),
 	(r'ALWAYS_DETAILED_SEC', False),
@@ -134,6 +135,7 @@ _doxygen_overrides = (
 	(r'SORT_MEMBERS_CTORS_1ST', True),
 	(r'SOURCE_BROWSER', False),
 	(r'STRICT_PROTO_MATCHING', False),
+	(r'STRIP_FROM_INC_PATH', None),  # we handle this
 	(r'SUBGROUPING', True),
 	(r'TAB_SIZE', 4),
 	(r'TOC_INCLUDE_HEADINGS', 3),
@@ -142,6 +144,7 @@ _doxygen_overrides = (
 	(r'USE_HTAGS', False),
 	(r'USE_MDFILE_AS_MAINPAGE', None),
 	(r'VERBATIM_HEADERS', False),
+	(r'WARN_AS_ERROR', False),  # we handle this
 	(r'WARN_IF_DOC_ERROR', True),
 	(r'WARN_IF_INCOMPLETE_DOC', True),
 	(r'WARN_LOGFILE', None),
@@ -156,11 +159,7 @@ def preprocess_doxyfile(context: Context):
 	assert isinstance(context, Context)
 
 	with doxygen.Doxyfile(
-		input_path=None,
-		output_path=context.doxyfile_path,
-		cwd=context.input_dir,
-		logger=context.verbose_logger,
-		doxygen_path=context.doxygen_path
+		input_path=None, output_path=context.doxyfile_path, cwd=context.input_dir, logger=context.verbose_logger
 	) as df:
 
 		df.append()
@@ -168,17 +167,16 @@ def preprocess_doxyfile(context: Context):
 		df.append(r'# marzer/poxy')
 		df.append(r'#---------------------------------------------------------------------------', end='\n\n')
 
-		df.append(r'# doxygen default overrides', end='\n\n')  # ----------------------------------------
+		df.append(r'# doxygen defaults', end='\n\n')  # ----------------------------------------
 
-		global _doxygen_overrides
-		for k, v in _doxygen_overrides:
+		for k, v in DOXYGEN_DEFAULTS:
 			df.set_value(k, v)
 
 		df.append()
 		df.append(r'# general config', end='\n\n')  # ---------------------------------------------------
 
 		df.set_value(r'OUTPUT_DIRECTORY', context.output_dir)
-		df.set_value(r'XML_OUTPUT', context.xml_dir)
+		df.set_value(r'XML_OUTPUT', context.temp_xml_dir)
 		df.set_value(r'PROJECT_NAME', context.name)
 		df.set_value(r'PROJECT_BRIEF', context.description)
 		df.set_value(r'PROJECT_LOGO', context.logo)
@@ -188,12 +186,11 @@ def preprocess_doxyfile(context: Context):
 			r'ENABLED_SECTIONS', (r'private', r'internal') if context.internal_docs else (r'public', r'external')
 		)
 		df.add_value(r'ENABLED_SECTIONS', r'poxy_supports_concepts')
+		if context.xml_v2:
+			df.set_value(r'INLINE_INHERITED_MEMB', False)
 
 		if context.generate_tagfile:
-			context.tagfile_path = Path(
-				context.output_dir, rf'{context.name.replace(" ","_")}.tagfile.xml' if context.name else r'tagfile.xml'
-			)
-			df.set_value(r'GENERATE_TAGFILE', context.tagfile_path.name)
+			df.set_value(r'GENERATE_TAGFILE', context.tagfile_path)
 		else:
 			df.set_value(r'GENERATE_TAGFILE', None)
 
@@ -216,7 +213,6 @@ def preprocess_doxyfile(context: Context):
 		df.append(r'# context.warnings', end='\n\n')  # ---------------------------------------------------
 
 		df.set_value(r'WARNINGS', context.warnings.enabled)
-		df.set_value(r'WARN_AS_ERROR', False)  # we do this ourself
 		df.set_value(r'WARN_IF_UNDOCUMENTED', context.warnings.undocumented)
 
 		df.append()
@@ -281,12 +277,10 @@ def preprocess_changelog(context: Context):
 	text = text.replace(r'&amp;', r'__poxy_thiswasan_amp')
 	text = text.replace(r'&#xFE0F;', r'__poxy_thiswasan_fe0f')
 	text = text.replace(r'@', r'__poxy_thiswasan_at')
-	if text.find(r'@tableofcontents') == -1 and text.find('\\tableofcontents') == -1 and text.find(r'[TOC]') == -1:
-		#text = f'[TOC]\n\n{text}'
-		nlnl = text.find(r'\n\n')
-		if nlnl != -1:
-			text = f'{text[:nlnl]}\n\n\\tableofcontents\n\n{text[nlnl:]}'
-		pass
+	text = f'\n{text}\n'
+	text = re.sub('\n#[^#].+?\n', '\n', text)
+	text = f'@page poxy_changelog Changelog\n\n@tableofcontents\n\n{text}'
+	text = text.rstrip()
 	text += '\n\n'
 	context.verbose(rf'Writing {context.changelog}')
 	with open(context.changelog, r'w', encoding=r'utf-8', newline='\n') as f:
@@ -315,7 +309,7 @@ def postprocess_xml(context: Context):
 	assert context is not None
 	assert isinstance(context, Context)
 
-	xml_files = get_all_files(context.xml_dir, any=(r'*.xml'))
+	xml_files = get_all_files(context.temp_xml_dir, any=(r'*.xml'))
 	if not xml_files:
 		return
 
@@ -360,8 +354,8 @@ def postprocess_xml(context: Context):
 			for (ip, ifn, iid) in hdata[3]:
 				implementation_header_mappings[iid] = hdata
 
-	setattr(context, r'compound_pages', dict())
-	setattr(context, r'compound_kinds', set())
+	context.compound_pages = dict()
+	context.compound_kinds = set()
 
 	# process xml files
 	if 1:
@@ -371,14 +365,14 @@ def postprocess_xml(context: Context):
 			# 'file' entries for markdown and dox files
 			dox_files = [rf'*{doxygen.mangle_name(ext)}.xml' for ext in (r'.dox', r'.md')]
 			dox_files.append(r'md_home.xml')
-			for xml_file in get_all_files(context.xml_dir, any=dox_files):
+			for xml_file in get_all_files(context.temp_xml_dir, any=dox_files):
 				delete_file(xml_file, logger=context.verbose_logger)
 
 			# 'dir' entries for empty directories
 			deleted = True
 			while deleted:
 				deleted = False
-				for xml_file in get_all_files(context.xml_dir, all=(r'dir*.xml')):
+				for xml_file in get_all_files(context.temp_xml_dir, all=(r'dir*.xml')):
 					xml = etree.parse(str(xml_file), parser=xml_parser)
 					compounddef = xml.getroot().find(r'compounddef')
 					if compounddef is None or compounddef.get(r'kind') != r'dir':
@@ -386,7 +380,7 @@ def postprocess_xml(context: Context):
 					existing_inners = 0
 					for subtype in (r'innerfile', r'innerdir'):
 						for inner in compounddef.findall(subtype):
-							ref_file = Path(context.xml_dir, rf'{inner.get(r"refid")}.xml')
+							ref_file = Path(context.temp_xml_dir, rf'{inner.get(r"refid")}.xml')
 							if ref_file.exists():
 								existing_inners = existing_inners + 1
 					if not existing_inners:
@@ -397,7 +391,7 @@ def postprocess_xml(context: Context):
 		tentative_macros = regex_or(context.code_blocks.macros)
 		macros = set()
 		cpp_tree = CppTree()
-		xml_files = get_all_files(context.xml_dir, any=(r'*.xml'))
+		xml_files = get_all_files(context.temp_xml_dir, any=(r'*.xml'))
 		tagfiles = [f for _, (f, _) in context.tagfiles.items()]
 		xml_files = xml_files + tagfiles
 		all_inners_by_type = {r'namespace': set(), r'class': set(), r'concept': set()}
@@ -418,7 +412,7 @@ def postprocess_xml(context: Context):
 				for compound in [
 					tag for tag in root.findall(r'compound') if tag.get(r'kind') in (r'file', r'dir', r'concept')
 				]:
-					ref_file = Path(context.xml_dir, rf'{compound.get(r"refid")}.xml')
+					ref_file = Path(context.temp_xml_dir, rf'{compound.get(r"refid")}.xml')
 					if not ref_file.exists():
 						root.remove(compound)
 						changed = True
@@ -571,15 +565,43 @@ def postprocess_xml(context: Context):
 									changed = True
 									break
 
-						# fix functions where keywords like 'friend' have been erroneously included in the return type
+						# fixes for functions:
+						# - goofy parsing of trailing return types
+						# - keywords like 'friend' erroneously included in the return type
 						if 1:
+
 							members = [
 								m for m in section.findall(r'memberdef') if m.get(r'kind') in (r'friend', r'function')
 							]
-							attribute_keywords = ((r'constexpr', r'constexpr',
-								r'yes'), (r'consteval', r'consteval', r'yes'), (r'explicit', r'explicit', r'yes'),
-								(r'static', r'static', r'yes'), (r'friend', None, None), (r'inline', r'inline',
-								r'yes'), (r'virtual', r'virt', r'virtual'))
+
+							# trailing return type bug (https://github.com/mosra/m.css/issues/94)
+							for member in members:
+								type_elem = member.find(r'type')
+								if type_elem is None or type_elem.text != r'auto':
+									continue
+								args_elem = member.find(r'argsstring')
+								if args_elem is None or not args_elem.text or args_elem.text.find(r'decltype') != -1:
+									continue
+								match = re.search(r'^(.*?)\s*->\s*([a-zA-Z][a-zA-Z0-9_::*&<>\s]+?)\s*$', args_elem.text)
+								if match:
+									args_elem.text = str(match[1])
+									trailing_return_type = str(match[2]).strip()
+									trailing_return_type = re.sub(r'\s+', r' ', trailing_return_type)
+									trailing_return_type = re.sub(r'(::|[<>*&])\s+', r'\1', trailing_return_type)
+									trailing_return_type = re.sub(r'\s+(::|[<>*&])', r'\1', trailing_return_type)
+									type_elem.text = trailing_return_type
+									changed = True
+
+							# leaked keywords
+							attribute_keywords = (
+								(r'constexpr', r'constexpr', r'yes'),  #
+								(r'consteval', r'consteval', r'yes'),
+								(r'explicit', r'explicit', r'yes'),
+								(r'static', r'static', r'yes'),
+								(r'friend', None, None),
+								(r'inline', r'inline', r'yes'),
+								(r'virtual', r'virt', r'virtual')
+							)
 							for member in members:
 								type = member.find(r'type')
 								if type is None or type.text is None:
@@ -662,12 +684,11 @@ def postprocess_xml(context: Context):
 				# files
 				if compound_kind == r'file':
 
-					# simplify the XML by removing junk not used by mcss
-					if not context.xml_only:
-						for tag in (r'includes', r'includedby', r'incdepgraph', r'invincdepgraph'):
-							for t in compounddef.findall(tag):
-								compounddef.remove(t)
-								changed = True
+					# simplify the XML by removing unnecessary junk
+					for tag in (r'includes', r'includedby', r'incdepgraph', r'invincdepgraph'):
+						for t in compounddef.findall(tag):
+							compounddef.remove(t)
+							changed = True
 
 					# get any macros for the syntax highlighter
 					for sectiondef in [
@@ -750,7 +771,7 @@ def postprocess_xml(context: Context):
 			for ns, vals in outer_namespaces.items():
 				xml_file = None
 				for outer_type in (r'namespace', r'struct', r'class', r'union'):
-					f = Path(context.xml_dir, rf'{outer_type}{doxygen.mangle_name(ns)}.xml')
+					f = Path(context.temp_xml_dir, rf'{outer_type}{doxygen.mangle_name(ns)}.xml')
 					if f.exists():
 						xml_file = f
 						break
@@ -781,7 +802,7 @@ def postprocess_xml(context: Context):
 		# merge extracted implementations
 		if extracted_implementation:
 			for (hp, hfn, hid, impl) in implementation_header_data:
-				xml_file = Path(context.xml_dir, rf'{hid}.xml')
+				xml_file = Path(context.temp_xml_dir, rf'{hid}.xml')
 				context.verbose(rf'Merging implementation nodes into {xml_file}')
 				xml = etree.parse(str(xml_file), parser=xml_parser)
 				compounddef = xml.getroot().find(r'compounddef')
@@ -843,11 +864,11 @@ def postprocess_xml(context: Context):
 	if 1 and context.implementation_headers:
 		for hdata in implementation_header_data:
 			for (ip, ifn, iid) in hdata[3]:
-				delete_file(Path(context.xml_dir, rf'{iid}.xml'), logger=context.verbose_logger)
+				delete_file(Path(context.temp_xml_dir, rf'{iid}.xml'), logger=context.verbose_logger)
 
 	# scan through the files and substitute impl header ids and paths as appropriate
 	if 1 and context.implementation_headers:
-		xml_files = get_all_files(context.xml_dir, any=('*.xml'))
+		xml_files = get_all_files(context.temp_xml_dir, any=('*.xml'))
 		for xml_file in xml_files:
 			context.verbose(rf"Re-linking implementation headers in '{xml_file}'")
 			xml_text = read_all_text_from_file(xml_file, logger=context.verbose_logger)
@@ -860,14 +881,44 @@ def postprocess_xml(context: Context):
 				xml = etree.parse(b, parser=xml_parser)
 				write_xml_to_file(xml, xml_file)
 
-	# convert the definition lists into compiled regexes since we've now extracted everything useful
+
+
+def postprocess_xml_v2(context: Context):
+	assert context is not None
+	assert isinstance(context, Context)
+
+	log_func = lambda m: context.verbose(m)
+
+	g = doxygen.read_graph_from_xml(context.temp_xml_dir, log_func=log_func)
+
+	# delete 'file' nodes for markdown and dox files
+	g.remove(filter=lambda n: n.type is graph.File and re.search(r'[.](?:md|dox)$', n.local_name, flags=re.I))
+
+	# delete empty 'dir' nodes
+	g.remove(filter=lambda n: n.type is graph.Directory and not len(list(n(graph.File, graph.Directory))))
+
+	# todo:
+	# - extract namespaces, types and enum values for syntax highlighting
+	# - enumerate all compound pages and their types for later (e.g. HTML post-process)
+	# - merge user-defined sections with the same name
+	# - sort user-defined sections based on their name
+	# - implementation headers
+
+	for f in enumerate_files(context.temp_xml_dir, any=r'*.xml'):
+		delete_file(f, logger=log_func)
+	doxygen.write_graph_to_xml(g, context.temp_xml_dir, log_func=log_func)
+
+
+
+def compile_syntax_highlighter_regexes(context: Context):
+	assert context is not None
+	assert isinstance(context, Context)
+
 	context.code_blocks.namespaces = regex_or(
 		context.code_blocks.namespaces, pattern_prefix='(?:::)?', pattern_suffix='(?:::)?'
 	)
 	context.code_blocks.types = regex_or(context.code_blocks.types, pattern_prefix='(?:::)?', pattern_suffix='(?:::)?')
 	context.code_blocks.enums = regex_or(context.code_blocks.enums, pattern_prefix='(?:::)?')
-	context.code_blocks.string_literals = regex_or(context.code_blocks.string_literals)
-	context.code_blocks.numeric_literals = regex_or(context.code_blocks.numeric_literals)
 	context.code_blocks.macros = regex_or(context.code_blocks.macros)
 	context.autolinks = tuple([(re.compile('(?<![a-zA-Z_])' + expr + '(?![a-zA-Z_])'), uri)
 		for expr, uri in context.autolinks])
@@ -1154,6 +1205,7 @@ def postprocess_html(context: Context):
 
 	context.fixers = (
 		fixers.MarkTOC(),
+		fixers.Pygments(),
 		fixers.CodeBlocks(),
 		fixers.Banner(),
 		fixers.CPPModifiers1(),
@@ -1277,7 +1329,7 @@ def run_doxygen(context: Context):
 	assert isinstance(context, Context)
 	with make_temp_file() as stdout, make_temp_file() as stderr:
 		try:
-			subprocess.run([str(context.doxygen_path), str(context.doxyfile_path)],
+			subprocess.run([str(doxygen.path()), str(context.doxyfile_path)],
 				check=True,
 				stdout=stdout,
 				stderr=stderr,
@@ -1336,19 +1388,20 @@ def run_mcss(context: Context):
 
 
 def run(
-	config_path=None,
-	output_dir='.',
-	threads=-1,
-	cleanup=True,
-	verbose=False,
-	doxygen_path=None,
+	config_path: Path = None,
+	output_dir: Path = '.',
+	output_html: bool = True,
+	output_xml: bool = False,
+	threads: int = -1,
+	cleanup: bool = True,
+	verbose: bool = False,
 	logger=None,
-	xml_only=False,
-	html_include=None,
-	html_exclude=None,
-	treat_warnings_as_errors=None,
-	theme=None,
-	copy_assets=True
+	html_include: str = None,
+	html_exclude: str = None,
+	treat_warnings_as_errors: bool = None,
+	theme: str = None,
+	copy_assets: bool = True,
+	**kwargs
 ):
 
 	timer = lambda desc: ScopeTimer(desc, print_start=True, print_end=context.verbose_logger)
@@ -1356,55 +1409,74 @@ def run(
 	with Context(
 		config_path=config_path,
 		output_dir=output_dir,
+		output_html=output_html,
+		output_xml=output_xml,
 		threads=threads,
 		cleanup=cleanup,
 		verbose=verbose,
-		doxygen_path=doxygen_path,
 		logger=logger,
-		xml_only=xml_only,
 		html_include=html_include,
 		html_exclude=html_exclude,
 		treat_warnings_as_errors=treat_warnings_as_errors,
 		theme=theme,
-		copy_assets=copy_assets
+		copy_assets=copy_assets,
+		**kwargs
 	) as context:
 
 		preprocess_doxyfile(context)
 		preprocess_tagfiles(context)
 		preprocess_changelog(context)
 
-		# generate + postprocess XML
-		with timer(r'Generating XML files with Doxygen') as t:
-			run_doxygen(context)
-		with timer(r'Post-processing XML files') as t:
-			postprocess_xml(context)
-		if context.xml_only:
+		if not context.output_html and not context.output_xml:
 			return
 
-		# generate HTML with mcss
-		preprocess_mcss_config(context)
-		with timer(r'Generating HTML files with m.css') as t:
-			run_mcss(context)
-
-		# copy extra_files
-		with ScopeTimer(r'Copying extra_files', print_start=True, print_end=context.verbose_logger) as t:
-			for dest_name, source_path in context.extra_files.items():
-				dest_path = Path(context.html_dir, dest_name).resolve()
-				dest_path.parent.mkdir(exist_ok=True)
-				copy_file(source_path, dest_path, logger=context.verbose_logger)
-
-		# copy fonts
-		if context.copy_assets:
-			with ScopeTimer(r'Copying fonts', print_start=True, print_end=context.verbose_logger) as t:
-				copy_tree(str(dirs.FONTS), str(Path(context.assets_dir, r'fonts')))
-
-		# move the tagfile into the html directory
-		if context.generate_tagfile:
-			if context.tagfile_path.exists():
-				move_file(context.tagfile_path, Path(context.output_dir, r'html'), logger=context.verbose_logger)
+		# generate + postprocess XML in temp_xml_dir
+		# (we always do this even when output_xml is false because it is required by the html)
+		with timer(rf'Generating XML files with Doxygen {doxygen.version()}') as t:
+			run_doxygen(context)
+		with timer(r'Post-processing XML files') as t:
+			if context.xml_v2:
+				postprocess_xml_v2(context)
 			else:
-				context.warning(rf'Doxygen tagfile {context.tagfile_path} not found!')
+				postprocess_xml(context)
 
-		# post-process html files
-		with timer(r'Post-processing HTML files') as t:
-			postprocess_html(context)
+		# postprocess_xml extracts type information so now we can compile the highlighter regexes
+		compile_syntax_highlighter_regexes(context)
+
+		# XML (the user-requested copy)
+		if context.output_xml:
+
+			with ScopeTimer(r'Copying XML', print_start=True, print_end=context.verbose_logger) as t:
+				copy_tree(str(context.temp_xml_dir), str(context.xml_dir))
+
+			# copy tagfile
+			if context.generate_tagfile:
+				copy_file(context.tagfile_path, context.xml_dir, logger=context.verbose_logger)
+
+		# HTML
+		if context.output_html:
+
+			# generate HTML with mcss
+			preprocess_mcss_config(context)
+			with timer(r'Generating HTML files with m.css') as t:
+				run_mcss(context)
+
+			# copy extra_files
+			with ScopeTimer(r'Copying extra_files', print_start=True, print_end=context.verbose_logger) as t:
+				for dest_name, source_path in context.extra_files.items():
+					dest_path = Path(context.html_dir, dest_name).resolve()
+					dest_path.parent.mkdir(exist_ok=True)
+					copy_file(source_path, dest_path, logger=context.verbose_logger)
+
+			# copy fonts
+			if context.copy_assets:
+				with ScopeTimer(r'Copying fonts', print_start=True, print_end=context.verbose_logger) as t:
+					copy_tree(str(dirs.FONTS), str(Path(context.assets_dir, r'fonts')))
+
+			# copy tagfile
+			if context.generate_tagfile:
+				copy_file(context.tagfile_path, context.html_dir, logger=context.verbose_logger)
+
+			# post-process html files
+			with timer(r'Post-processing HTML files') as t:
+				postprocess_html(context)
